'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.toRes = toRes;
exports.getThemeOsmDiff = getThemeOsmDiff;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**	Creates a callback that proxies node callback style arguments to an Express Response object.
 *	@param {express.Response} res	Express HTTP Response
 *	@param {number} [status=200]	Status code to send on success
 *
 *	@example
 *		list(req, res) {
 *			collection.find({}, toRes(res));
 *		}
 */
function toRes(res) {
	var status = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 200;

	return function (err, thing) {
		if (err) return res.status(500).send(err);

		if (thing && typeof thing.toObject === 'function') {
			thing = thing.toObject();
		}
		res.status(status).json(thing);
	};
}

/**
 * Renvoie le json des thèmes avec tableau des groupes d'objets 
 * @param {*} db 
 * @param {*} theme 
 * @param {*} filter 
 */
function getThemeOsmDiff(db, theme, filter) {
	var Op = db.Sequelize.Op;
	var Setting = db.setting;
	var jTheme = theme.toJSON();
	var nbThemeDiff = 0;

	return Setting.findAll({
		where: {
			name: _defineProperty({}, Op.eq, 'objects_limit')
		}
	})
	// save/update user (return Promise) or status 404 (res)
	.then(function (res) {

		var settings = res.reduce(function (a, s) {
			a[s.name] = s.value;
			return a;
		}, {});

		var objectsLimit = settings.objects_limit ? settings.objects_limit : 15;

		return Promise.all(jTheme.objects.map(function (obj) {
			var _where;

			var sequelize = db.sequelize;
			var Validation = db.validation;
			var Contributor = db.contributor;
			var OsmDiff = db.osmdiff;

			//fonction de génération clause WHERE colonne contient (@>) hstore
			var hstoreContainHstore = function hstoreContainHstore(col, hArray) {
				return sequelize.where(sequelize.col(col), '@>', sequelize.fn('hstore', sequelize.literal('ARRAY[\'' + hArray.join('\',\'') + '\']')));
			};

			//fonction de génération clause WHERE colonne - opérateur - clés
			var hstoreContainKeys = function hstoreContainKeys(col, op, keys) {
				return sequelize.where(sequelize.col(col), op, sequelize.literal('ARRAY[\'' + keys.join('\',\'') + '\']'));
			};

			// Traduit une requête JSON en requête sequelize
			var translateQuery = function translateQuery(col, parsedQuery) {

				var firstOp = getFirstOperator(parsedQuery);
				if (Array.isArray(parsedQuery[firstOp]) && firstOp !== null) {
					var conditions = [];

					for (var i = 0; i < parsedQuery[firstOp].length; i++) {
						var node = parsedQuery[firstOp][i];
						var key = Object.keys(node)[0];
						var val = node[key];

						if (typeof val === 'string') {
							if (val === '*') {
								conditions.push(sequelize.where(sequelize.col(col), '?', '' + key));
							} else {
								conditions.push(sequelize.where(sequelize.col(col), '@>', '"' + key + '"=>"' + val + '"'));
							}
						} else if ((typeof val === 'undefined' ? 'undefined' : _typeof(val)) === 'object') {
							conditions.push(translateQuery(col, node));
						}
					}

					if (firstOp === '[op.or]') {
						return _defineProperty({}, Op.or, conditions);
					} else if (firstOp === '[op.and]') {
						return _defineProperty({}, Op.and, conditions);
					}
				}
			};

			// Retourne le premier opérateur présent
			var getFirstOperator = function getFirstOperator(parsedQuery) {
				if (parsedQuery['[op.or]']) {
					return '[op.or]';
				} else if (parsedQuery['[op.and]']) {
					return '[op.and]';
				} else {
					return false;
				}
			};

			// CLAUSE WHERE A CALCULER :
			// od_new_tags vérifie la requête du GO OR od_old_tags vérifie la requête du GO
			// AND
			// od_new_tags contient tous les tags obligatoires du GO OR od_old_tags contient tous les tags obligatoires du GO
			// AND
			// od_added_tags contient au moins 1 tag à valider OR od_deleted_tags contient au moins 1 tag à valider OR od_modified_tags contient au moins 1 tag à valider

			//clause WHERE col / requête du groupe d'objets (og_query)
			var compareQuery = function compareQuery(col) {
				var clauses = [];
				// Requête groupe d'objet
				if (obj.query_json) {
					var query = translateQuery(col, obj.query_json);
					clauses.push(query);
				}
				// Si query est vide, on crée une clause WHERE toujours vraie à la place de hstoreContainHstore
				else {
						clauses.push(sequelize.where(sequelize.col(col), '=', sequelize.col(col)));
					}

				// clauses renvoie une clause WHERE pour la future requête
				return clauses;
			};

			//fonction de comparaison avec les tags obligatoires du groupe d'objets (og_tags)
			var compareTags = function compareTags(col) {
				var clauses = [];

				if (obj.keys.length) {
					var keys = obj.keys.map(function (k) {
						return k.name;
					});
					clauses.push(hstoreContainKeys(col, '?&', keys));
				}
				// Si keys est vide, on crée une clause WHERE toujours vraie à la place de hstoreContainKeys
				else {
						clauses.push(sequelize.where(sequelize.col(col), '=', sequelize.col(col)));
					}

				return clauses;
			};

			//fonction de comparaison avec les tags à valider (droits)
			var compareRights = function compareRights(col) {
				var clauses = [];

				if (obj.rights.length) {
					var keys = obj.rights.reduce(function (a, r) {
						return [].concat(a, r.keys);
					}, []).map(function (k) {
						return k.name;
					});
					clauses.push(hstoreContainKeys(col, '?|', keys));
				}

				return clauses;
			};

			var newCompareQuery = compareQuery('od_new_tags');
			var oldCompareQuery = compareQuery('od_old_tags');
			var newCompareTags = compareTags('od_new_tags');
			var oldCompareTags = compareTags('od_old_tags');
			var addedCompareRights = compareRights('od_added_tags');
			var deletedCompareRights = compareRights('od_deleted_tags');
			var modifiedCompareRights = compareRights('od_modified_tags');

			// cherche les osmdiff
			return (

				// condition : si newCompareQuery n'est pas nul, alors db.osmdiff.findAll sinon : Promise.resolve([])
				// ((newCompareQuery.length) ? 

				db.osmdiff.findAll({
					where: (_where = {}, _defineProperty(_where, Op.and, [_defineProperty({}, Op.or, [_extends({}, newCompareQuery), _extends({}, oldCompareQuery)]), _defineProperty({}, Op.or, [_extends({}, newCompareTags), _extends({}, oldCompareTags)]), _defineProperty({}, Op.or, [_extends({}, addedCompareRights), _extends({}, deletedCompareRights), _extends({}, modifiedCompareRights)])]), _defineProperty(_where, '$status.st_name$', _defineProperty({}, Op.or, [_defineProperty({}, Op.in, ['tovalidate', 'waiting']), _defineProperty({}, Op.eq, null)])), _defineProperty(_where, '$validations.val_od_version$', _defineProperty({}, Op.or, [_defineProperty({}, Op.eq, sequelize.col('osmdiff.od_new_version')), _defineProperty({}, Op.eq, null)])), _where),
					order: [['name', 'ASC'], ['id', 'ASC']],
					include: [{
						association: OsmDiff.NewChangeset
						// attributes: ['ocs_id'],
					}, {
						association: OsmDiff.Status,
						required: false
						// attributes: ['st_name'],
					}, {
						association: OsmDiff.Validation,
						required: false,
						include: [{
							association: Validation.Status
						}, {
							association: Validation.User
							// attributes: ['u_id', 'u_login', 'u_first_name', 'u_last_name'],
						}]
					}, {
						association: OsmDiff.Contributor,
						// attributes: ['ct_name'],
						include: [{
							association: Contributor.Vigilance
							// attributes: ['vg_color'],
						}]
					}]
					// limit: sequelize.literal(`(SELECT CAST(s_value AS integer) FROM lebontag.lbt_setting WHERE s_name = 'objects_limit' )`),
					// subQuery: false,
				}).then(function (osmdiff) {
					return Promise.resolve(osmdiff.filter(function (o) {
						// filter by validations for the associated rights
						var osmKeys = Object.keys(_extends({}, o.new_tags, o.old_tags));
						// validations are valid only because osm object is to validate
						var validatedKeys = o.validations.filter(function (v) {
							return v.status.name === 'valid';
						}).reduce(function (a, v) {
							return [].concat(a, v.tags);
						}, []);
						var rightKeys = obj.rights.reduce(function (a, r) {
							return [].concat(a, r.keys);
						}, []).map(function (k) {
							return k.name;
						});

						// find keys to validate i.e. osm keys that exist in right
						var keysToValidate = osmKeys.filter(function (k) {
							return rightKeys.includes(k);
						});

						// find remaining keys to validate i.e. keys that are not valid yet
						var remainingKeys = keysToValidate.filter(function (k) {
							return !validatedKeys.includes(k);
						});

						// is there some keys?
						return !!remainingKeys.length;
					})
					// Filtre passé par l'URL
					.filter(function (osm) {
						try {
							filter = filter ? filter : {};
							filter.contributors = filter.contributors ? filter.contributors : [];
							filter.vigilance = filter.vigilance ? filter.vigilance : {};
							return (!filter.contributors.length || osm.contributor && filter.contributors.some(function (c) {
								return c.id === osm.contributor.id;
							})) && (!filter.vigilance.id || osm.contributor && osm.contributor.vigilance && filter.vigilance.id === osm.contributor.vigilance.id);
						} catch (e) {
							return true;
						}
					}));
				})
				//: Promise.resolve([])
				.then(function (diff) {
					var nbOsmDiff = diff.length;
					nbThemeDiff = nbThemeDiff + nbOsmDiff;
					if (diff.length > objectsLimit) {
						diff.length = objectsLimit;
					}
					return Promise.resolve(_extends({}, obj, {
						nbOsmDiff: nbOsmDiff,
						osmdiff: diff.map(function (d) {

							// Ajout de l'id du groupe
							d.dataValues.og_gr_id = obj.id;

							// Retour
							return d.toJSON();
						})
					}));
				}).catch(function (err) {
					console.log('err', err);
					throw err;
				})
			);
		})).then(function (objects) {
			return Promise.resolve(_extends({}, jTheme, {
				objects: objects,
				nbThemeDiff: nbThemeDiff
			}));
		}).catch(function (err) {
			console.log('err', err);
			throw err;
		});
	});
}