#!/usr/bin/python3
# -*- encoding: UTF-8 -*-

# Imports
try:
    import yaml, os, requests, psycopg2, gzip, time, logging, sys, re, subprocess, shutil, platform
    from pathlib import Path
    from datetime import date, datetime, timezone
    from copy import deepcopy
    from psycopg2 import sql
    from psycopg2.extensions import AsIs
    from lxml import etree as ET
except Exception as e:
    print(e)
    sys.exit(1)

# Paramètres config.yaml
try:
    with open("../config.yaml", 'rt', encoding='UTF-8') as yml:
        config = yaml.safe_load(yml)
        lbtHost = config['database']['host']
        lbtPort = config['database']['port']
        lbtDb = config['database']['dbname']
        lbtUser = config['database']['username']
        lbtPassword = config['database']['password']
        lbtSchema = config['database']['schema']
        lbtSrid = int(config['database']['srid'])
except Exception as e:
    print(e)
    sys.exit(1)

# Date actuelle
# 2020-01-16 09:18:39
end_date = datetime.utcnow().replace(microsecond=0)
end_date_str = end_date.strftime("%Y%m%d")

# Fichier journal (DEBUG/INFO/WARNING/ERROR/CRITICAL)
try:
    logDirectory = '../log/lbt-autovalidation/' + str(end_date.year) + str('%02d' % end_date.month) + '/'
    if not os.path.exists(logDirectory):
        os.makedirs(logDirectory)
    logFilename = logDirectory + 'autovalidation_' + end_date_str + '.log'
    logging.basicConfig(filename=logFilename,filemode='w',format='%(asctime)s|%(levelname)s|%(message)s',datefmt='%d/%m/%Y %H:%M:%S',level=logging.INFO)
    logging.info('Start Python script.')
except Exception as e:
    print(e)
    sys.exit(1)

# Ouvre la connexion PG sur LeBonTag
try:
    logging.info('Open database connection.')
    connect_str = "host=%s port=%s dbname=%s user=%s password=%s" % (lbtHost,lbtPort,lbtDb,lbtUser,lbtPassword)
    conn = psycopg2.connect(connect_str)
    cursor = conn.cursor()
except Exception as e:
    logging.critical('PG connection error : ' + str(e))
    sys.exit(1)

# Passe tous les objets osmdiff actuellement en l'état 'tovalidate' à l'état 'valid' pour les contributeurs à validation automatique
try:
    logging.info('Update lbt_osmdiff with autovalidated contributors...')
    cursor.execute(sql.SQL("UPDATE {0}.lbt_osmdiff SET od_status = (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'valid') WHERE od_new_ct_id IN (SELECT ct_id FROM {0}.lbt_contributor WHERE ct_auto_validation is true) AND od_status = (SELECT st_id FROM {0}.lbt_status WHERE st_name = 'tovalidate');").format(sql.Identifier(lbtSchema)))
    updated_rows = cursor.rowcount
    conn.commit()
    cursor.close()
    conn.close()
    logging.info(str(updated_rows) + ' row(s) updated.')
except Exception as e:
    logging.critical('Autovalidation (SQL UPDATE) error : ' + str(e))
    sys.exit(1)

logging.info('Python script ended.')
